# This file was auto-generated by salt's setup on Thursday, 13 August 2015 @ 20:08:57 UTC.

from salt.version import SaltStackVersion

__saltstack_version__ = SaltStackVersion(2015, 5, 4, 0, 0, 0, None)
